require 'spec_helper'

require 'dome/view_methods'
require 'dome/errors'

describe 'ViewMethods' do
  subject do
    Class.new do
      include Dome::ViewMethods

      def initialize
        @num = 123
        @float = 1.5
        @symbols = [:l, :m, :n]
        @words = ["abc", "xyz"]
      end
    end.new
  end

  describe "#present" do
    class ThingPresenter < Dome::Presenter; end
    let(:set_presenter) do
      subject.instance_eval { present(:thing, @num) }
    end

    it "raises error if presenter class is absent" do
      expect {
        subject.instance_eval { present(:anything, @num) }
      }.to raise_error(Dome::PresenterClassNotFound)
    end

    it "sets @view_context variable to subject" do
      expect(set_presenter.instance_variable_get(:@view_context)).to eq subject
    end

    it "calls a defined method" do
      expect(set_presenter).to be_a ThingPresenter
    end
  end

  describe "#exhibit" do
    class FixnumExhibit < Dome::Exhibit; end
    let(:set_exhibit) do
      subject.instance_eval { exhibit(@num) }
    end

    it "raises error if exhibit class is absent" do
      expect {
        subject.instance_eval { exhibit(@float) }
      }.to raise_error(Dome::ExhibitClassNotFound)
    end

    it "returns a FixnumExhibit instance" do
      expect(
        set_exhibit
      ).to be_a FixnumExhibit
    end

    it "sets @view_context variable to subject" do
      expect(
        set_exhibit.instance_variable_get(:@view_context)
      ).to eq subject
    end
  end

  describe "#exhibit_enum" do
    class StringExhibit < Dome::Exhibit; end

    it "raises error if exhibit class is absent" do
      expect {
        subject.instance_eval { exhibit_enum(@symbols) }
      }.to raise_error(Dome::ExhibitClassNotFound)
    end

    it "returns a StringExhibit instances" do
      ary = subject.instance_eval { exhibit_enum(@words) }
      expect(ary.first).to be_a StringExhibit
      expect(ary.second).to be_a StringExhibit
    end
  end
end
